/*
 * This file is part of ViaLegacy - https://github.com/RaphiMC/ViaLegacy
 * Copyright (C) 2023 RK_01/RaphiMC and contributors
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.raphimc.vialegacy.protocols.release.protocol1_7_2_5to1_6_4.rewriter;

import com.viaversion.viaversion.libs.fastutil.ints.Int2ObjectMap;
import com.viaversion.viaversion.libs.fastutil.ints.Int2ObjectOpenHashMap;

public class StatisticRewriter {

    private static final Int2ObjectMap<String> STATISTICS = new Int2ObjectOpenHashMap<>(764, 0.99F);

    static {
        STATISTICS.put(1000, "stat.startGame");
        STATISTICS.put(1001, "stat.createWorld");
        STATISTICS.put(1002, "stat.loadWorld");
        STATISTICS.put(1003, "stat.joinMultiplayer");
        STATISTICS.put(1004, "stat.leaveGame");
        STATISTICS.put(1100, "stat.playOneMinute");
        STATISTICS.put(2000, "stat.walkOneCm");
        STATISTICS.put(2001, "stat.swimOneCm");
        STATISTICS.put(2002, "stat.fallOneCm");
        STATISTICS.put(2003, "stat.climbOneCm");
        STATISTICS.put(2004, "stat.flyOneCm");
        STATISTICS.put(2005, "stat.diveOneCm");
        STATISTICS.put(2006, "stat.minecartOneCm");
        STATISTICS.put(2007, "stat.boatOneCm");
        STATISTICS.put(2008, "stat.pigOneCm");
        STATISTICS.put(2010, "stat.jump");
        STATISTICS.put(2011, "stat.drop");
        STATISTICS.put(2020, "stat.damageDealt");
        STATISTICS.put(2021, "stat.damageTaken");
        STATISTICS.put(2022, "stat.deaths");
        STATISTICS.put(2023, "stat.mobKills");
        STATISTICS.put(2024, "stat.playerKills");
        STATISTICS.put(2025, "stat.fishCaught");
        STATISTICS.put(5242880, "achievement.openInventory");
        STATISTICS.put(5242881, "achievement.mineWood");
        STATISTICS.put(5242882, "achievement.buildWorkBench");
        STATISTICS.put(5242883, "achievement.buildPickaxe");
        STATISTICS.put(5242884, "achievement.buildFurnace");
        STATISTICS.put(5242885, "achievement.acquireIron");
        STATISTICS.put(5242886, "achievement.buildHoe");
        STATISTICS.put(5242887, "achievement.makeBread");
        STATISTICS.put(5242888, "achievement.bakeCake");
        STATISTICS.put(5242889, "achievement.buildBetterPickaxe");
        STATISTICS.put(5242890, "achievement.cookFish");
        STATISTICS.put(5242891, "achievement.onARail");
        STATISTICS.put(5242892, "achievement.buildSword");
        STATISTICS.put(5242893, "achievement.killEnemy");
        STATISTICS.put(5242894, "achievement.killCow");
        STATISTICS.put(5242895, "achievement.flyPig");
        STATISTICS.put(5242896, "achievement.snipeSkeleton");
        STATISTICS.put(5242897, "achievement.diamonds");
        STATISTICS.put(5242898, "achievement.portal");
        STATISTICS.put(5242899, "achievement.ghast");
        STATISTICS.put(5242900, "achievement.blazeRod");
        STATISTICS.put(5242901, "achievement.potion");
        STATISTICS.put(5242902, "achievement.theEnd");
        STATISTICS.put(5242903, "achievement.theEnd2");
        STATISTICS.put(5242904, "achievement.enchantments");
        STATISTICS.put(5242905, "achievement.overkill");
        STATISTICS.put(5242906, "achievement.bookcase");
        STATISTICS.put(16777217, "stat.mineBlock.1");
        STATISTICS.put(16777218, "stat.mineBlock.2");
        STATISTICS.put(16777219, "stat.mineBlock.3");
        STATISTICS.put(16777220, "stat.mineBlock.4");
        STATISTICS.put(16777221, "stat.mineBlock.5");
        STATISTICS.put(16777222, "stat.mineBlock.6");
        STATISTICS.put(16777228, "stat.mineBlock.12");
        STATISTICS.put(16777229, "stat.mineBlock.13");
        STATISTICS.put(16777230, "stat.mineBlock.14");
        STATISTICS.put(16777231, "stat.mineBlock.15");
        STATISTICS.put(16777232, "stat.mineBlock.16");
        STATISTICS.put(16777233, "stat.mineBlock.17");
        STATISTICS.put(16777234, "stat.mineBlock.18");
        STATISTICS.put(16777235, "stat.mineBlock.19");
        STATISTICS.put(16777236, "stat.mineBlock.20");
        STATISTICS.put(16777237, "stat.mineBlock.21");
        STATISTICS.put(16777238, "stat.mineBlock.22");
        STATISTICS.put(16777239, "stat.mineBlock.23");
        STATISTICS.put(16777240, "stat.mineBlock.24");
        STATISTICS.put(16777241, "stat.mineBlock.25");
        STATISTICS.put(16777243, "stat.mineBlock.27");
        STATISTICS.put(16777244, "stat.mineBlock.28");
        STATISTICS.put(16777245, "stat.mineBlock.29");
        STATISTICS.put(16777246, "stat.mineBlock.30");
        STATISTICS.put(16777247, "stat.mineBlock.31");
        STATISTICS.put(16777248, "stat.mineBlock.32");
        STATISTICS.put(16777249, "stat.mineBlock.33");
        STATISTICS.put(16777250, "stat.mineBlock.34");
        STATISTICS.put(16777251, "stat.mineBlock.35");
        STATISTICS.put(16777252, "stat.mineBlock.36");
        STATISTICS.put(16777253, "stat.mineBlock.37");
        STATISTICS.put(16777254, "stat.mineBlock.38");
        STATISTICS.put(16777255, "stat.mineBlock.39");
        STATISTICS.put(16777256, "stat.mineBlock.40");
        STATISTICS.put(16777257, "stat.mineBlock.41");
        STATISTICS.put(16777258, "stat.mineBlock.42");
        STATISTICS.put(16777259, "stat.mineBlock.43");
        STATISTICS.put(16777260, "stat.mineBlock.44");
        STATISTICS.put(16777261, "stat.mineBlock.45");
        STATISTICS.put(16777262, "stat.mineBlock.46");
        STATISTICS.put(16777263, "stat.mineBlock.47");
        STATISTICS.put(16777264, "stat.mineBlock.48");
        STATISTICS.put(16777265, "stat.mineBlock.49");
        STATISTICS.put(16777266, "stat.mineBlock.50");
        STATISTICS.put(16777269, "stat.mineBlock.53");
        STATISTICS.put(16777270, "stat.mineBlock.54");
        STATISTICS.put(16777272, "stat.mineBlock.56");
        STATISTICS.put(16777273, "stat.mineBlock.57");
        STATISTICS.put(16777274, "stat.mineBlock.58");
        STATISTICS.put(16777276, "stat.mineBlock.60");
        STATISTICS.put(16777277, "stat.mineBlock.61");
        STATISTICS.put(16777278, "stat.mineBlock.62");
        STATISTICS.put(16777281, "stat.mineBlock.65");
        STATISTICS.put(16777282, "stat.mineBlock.66");
        STATISTICS.put(16777283, "stat.mineBlock.67");
        STATISTICS.put(16777285, "stat.mineBlock.69");
        STATISTICS.put(16777286, "stat.mineBlock.70");
        STATISTICS.put(16777288, "stat.mineBlock.72");
        STATISTICS.put(16777289, "stat.mineBlock.73");
        STATISTICS.put(16777290, "stat.mineBlock.74");
        STATISTICS.put(16777291, "stat.mineBlock.75");
        STATISTICS.put(16777292, "stat.mineBlock.76");
        STATISTICS.put(16777293, "stat.mineBlock.77");
        STATISTICS.put(16777294, "stat.mineBlock.78");
        STATISTICS.put(16777295, "stat.mineBlock.79");
        STATISTICS.put(16777296, "stat.mineBlock.80");
        STATISTICS.put(16777297, "stat.mineBlock.81");
        STATISTICS.put(16777298, "stat.mineBlock.82");
        STATISTICS.put(16777300, "stat.mineBlock.84");
        STATISTICS.put(16777301, "stat.mineBlock.85");
        STATISTICS.put(16777302, "stat.mineBlock.86");
        STATISTICS.put(16777303, "stat.mineBlock.87");
        STATISTICS.put(16777304, "stat.mineBlock.88");
        STATISTICS.put(16777305, "stat.mineBlock.89");
        STATISTICS.put(16777306, "stat.mineBlock.90");
        STATISTICS.put(16777307, "stat.mineBlock.91");
        STATISTICS.put(16777311, "stat.mineBlock.95");
        STATISTICS.put(16777313, "stat.mineBlock.97");
        STATISTICS.put(16777314, "stat.mineBlock.98");
        STATISTICS.put(16777315, "stat.mineBlock.99");
        STATISTICS.put(16777316, "stat.mineBlock.100");
        STATISTICS.put(16777317, "stat.mineBlock.101");
        STATISTICS.put(16777318, "stat.mineBlock.102");
        STATISTICS.put(16777319, "stat.mineBlock.103");
        STATISTICS.put(16777320, "stat.mineBlock.104");
        STATISTICS.put(16777321, "stat.mineBlock.105");
        STATISTICS.put(16777322, "stat.mineBlock.106");
        STATISTICS.put(16777323, "stat.mineBlock.107");
        STATISTICS.put(16777324, "stat.mineBlock.108");
        STATISTICS.put(16777325, "stat.mineBlock.109");
        STATISTICS.put(16777326, "stat.mineBlock.110");
        STATISTICS.put(16777327, "stat.mineBlock.111");
        STATISTICS.put(16777328, "stat.mineBlock.112");
        STATISTICS.put(16777329, "stat.mineBlock.113");
        STATISTICS.put(16777330, "stat.mineBlock.114");
        STATISTICS.put(16777331, "stat.mineBlock.115");
        STATISTICS.put(16777332, "stat.mineBlock.116");
        STATISTICS.put(16777333, "stat.mineBlock.117");
        STATISTICS.put(16777334, "stat.mineBlock.118");
        STATISTICS.put(16777335, "stat.mineBlock.119");
        STATISTICS.put(16777336, "stat.mineBlock.120");
        STATISTICS.put(16777337, "stat.mineBlock.121");
        STATISTICS.put(16777338, "stat.mineBlock.122");
        STATISTICS.put(16777339, "stat.mineBlock.123");
        STATISTICS.put(16777340, "stat.mineBlock.124");
        STATISTICS.put(16777341, "stat.mineBlock.125");
        STATISTICS.put(16777342, "stat.mineBlock.126");
        STATISTICS.put(16777343, "stat.mineBlock.127");
        STATISTICS.put(16777344, "stat.mineBlock.128");
        STATISTICS.put(16777345, "stat.mineBlock.129");
        STATISTICS.put(16777346, "stat.mineBlock.130");
        STATISTICS.put(16777347, "stat.mineBlock.131");
        STATISTICS.put(16777348, "stat.mineBlock.132");
        STATISTICS.put(16777349, "stat.mineBlock.133");
        STATISTICS.put(16777350, "stat.mineBlock.134");
        STATISTICS.put(16777351, "stat.mineBlock.135");
        STATISTICS.put(16777352, "stat.mineBlock.136");
        STATISTICS.put(16777353, "stat.mineBlock.137");
        STATISTICS.put(16777354, "stat.mineBlock.138");
        STATISTICS.put(16777355, "stat.mineBlock.139");
        STATISTICS.put(16777356, "stat.mineBlock.140");
        STATISTICS.put(16777359, "stat.mineBlock.143");
        STATISTICS.put(16777360, "stat.mineBlock.144");
        STATISTICS.put(16777361, "stat.mineBlock.145");
        STATISTICS.put(16777362, "stat.mineBlock.146");
        STATISTICS.put(16777363, "stat.mineBlock.147");
        STATISTICS.put(16777364, "stat.mineBlock.148");
        STATISTICS.put(16777367, "stat.mineBlock.151");
        STATISTICS.put(16777368, "stat.mineBlock.152");
        STATISTICS.put(16777369, "stat.mineBlock.153");
        STATISTICS.put(16777370, "stat.mineBlock.154");
        STATISTICS.put(16777371, "stat.mineBlock.155");
        STATISTICS.put(16777372, "stat.mineBlock.156");
        STATISTICS.put(16777373, "stat.mineBlock.157");
        STATISTICS.put(16777374, "stat.mineBlock.158");
        STATISTICS.put(16777375, "stat.mineBlock.159");
        STATISTICS.put(16777386, "stat.mineBlock.170");
        STATISTICS.put(16777387, "stat.mineBlock.171");
        STATISTICS.put(16777388, "stat.mineBlock.172");
        STATISTICS.put(16777389, "stat.mineBlock.173");
        STATISTICS.put(16842753, "stat.craftItem.1");
        STATISTICS.put(16842757, "stat.craftItem.5");
        STATISTICS.put(16842772, "stat.craftItem.20");
        STATISTICS.put(16842774, "stat.craftItem.22");
        STATISTICS.put(16842775, "stat.craftItem.23");
        STATISTICS.put(16842776, "stat.craftItem.24");
        STATISTICS.put(16842777, "stat.craftItem.25");
        STATISTICS.put(16842779, "stat.craftItem.27");
        STATISTICS.put(16842780, "stat.craftItem.28");
        STATISTICS.put(16842781, "stat.craftItem.29");
        STATISTICS.put(16842785, "stat.craftItem.33");
        STATISTICS.put(16842787, "stat.craftItem.35");
        STATISTICS.put(16842793, "stat.craftItem.41");
        STATISTICS.put(16842794, "stat.craftItem.42");
        STATISTICS.put(16842796, "stat.craftItem.44");
        STATISTICS.put(16842797, "stat.craftItem.45");
        STATISTICS.put(16842798, "stat.craftItem.46");
        STATISTICS.put(16842799, "stat.craftItem.47");
        STATISTICS.put(16842802, "stat.craftItem.50");
        STATISTICS.put(16842805, "stat.craftItem.53");
        STATISTICS.put(16842806, "stat.craftItem.54");
        STATISTICS.put(16842809, "stat.craftItem.57");
        STATISTICS.put(16842810, "stat.craftItem.58");
        STATISTICS.put(16842813, "stat.craftItem.61");
        STATISTICS.put(16842817, "stat.craftItem.65");
        STATISTICS.put(16842818, "stat.craftItem.66");
        STATISTICS.put(16842819, "stat.craftItem.67");
        STATISTICS.put(16842821, "stat.craftItem.69");
        STATISTICS.put(16842822, "stat.craftItem.70");
        STATISTICS.put(16842824, "stat.craftItem.72");
        STATISTICS.put(16842828, "stat.craftItem.76");
        STATISTICS.put(16842829, "stat.craftItem.77");
        STATISTICS.put(16842830, "stat.craftItem.78");
        STATISTICS.put(16842832, "stat.craftItem.80");
        STATISTICS.put(16842834, "stat.craftItem.82");
        STATISTICS.put(16842836, "stat.craftItem.84");
        STATISTICS.put(16842837, "stat.craftItem.85");
        STATISTICS.put(16842841, "stat.craftItem.89");
        STATISTICS.put(16842843, "stat.craftItem.91");
        STATISTICS.put(16842848, "stat.craftItem.96");
        STATISTICS.put(16842850, "stat.craftItem.98");
        STATISTICS.put(16842853, "stat.craftItem.101");
        STATISTICS.put(16842854, "stat.craftItem.102");
        STATISTICS.put(16842855, "stat.craftItem.103");
        STATISTICS.put(16842859, "stat.craftItem.107");
        STATISTICS.put(16842860, "stat.craftItem.108");
        STATISTICS.put(16842861, "stat.craftItem.109");
        STATISTICS.put(16842864, "stat.craftItem.112");
        STATISTICS.put(16842865, "stat.craftItem.113");
        STATISTICS.put(16842866, "stat.craftItem.114");
        STATISTICS.put(16842868, "stat.craftItem.116");
        STATISTICS.put(16842875, "stat.craftItem.123");
        STATISTICS.put(16842878, "stat.craftItem.126");
        STATISTICS.put(16842880, "stat.craftItem.128");
        STATISTICS.put(16842882, "stat.craftItem.130");
        STATISTICS.put(16842883, "stat.craftItem.131");
        STATISTICS.put(16842885, "stat.craftItem.133");
        STATISTICS.put(16842886, "stat.craftItem.134");
        STATISTICS.put(16842887, "stat.craftItem.135");
        STATISTICS.put(16842888, "stat.craftItem.136");
        STATISTICS.put(16842890, "stat.craftItem.138");
        STATISTICS.put(16842891, "stat.craftItem.139");
        STATISTICS.put(16842895, "stat.craftItem.143");
        STATISTICS.put(16842897, "stat.craftItem.145");
        STATISTICS.put(16842898, "stat.craftItem.146");
        STATISTICS.put(16842899, "stat.craftItem.147");
        STATISTICS.put(16842900, "stat.craftItem.148");
        STATISTICS.put(16842903, "stat.craftItem.151");
        STATISTICS.put(16842904, "stat.craftItem.152");
        STATISTICS.put(16842906, "stat.craftItem.154");
        STATISTICS.put(16842907, "stat.craftItem.155");
        STATISTICS.put(16842908, "stat.craftItem.156");
        STATISTICS.put(16842909, "stat.craftItem.157");
        STATISTICS.put(16842910, "stat.craftItem.158");
        STATISTICS.put(16842911, "stat.craftItem.159");
        STATISTICS.put(16842922, "stat.craftItem.170");
        STATISTICS.put(16842923, "stat.craftItem.171");
        STATISTICS.put(16842924, "stat.craftItem.172");
        STATISTICS.put(16842925, "stat.craftItem.173");
        STATISTICS.put(16843008, "stat.craftItem.256");
        STATISTICS.put(16843009, "stat.craftItem.257");
        STATISTICS.put(16843010, "stat.craftItem.258");
        STATISTICS.put(16843011, "stat.craftItem.259");
        STATISTICS.put(16843013, "stat.craftItem.261");
        STATISTICS.put(16843014, "stat.craftItem.262");
        STATISTICS.put(16843015, "stat.craftItem.263");
        STATISTICS.put(16843016, "stat.craftItem.264");
        STATISTICS.put(16843017, "stat.craftItem.265");
        STATISTICS.put(16843018, "stat.craftItem.266");
        STATISTICS.put(16843019, "stat.craftItem.267");
        STATISTICS.put(16843020, "stat.craftItem.268");
        STATISTICS.put(16843021, "stat.craftItem.269");
        STATISTICS.put(16843022, "stat.craftItem.270");
        STATISTICS.put(16843023, "stat.craftItem.271");
        STATISTICS.put(16843024, "stat.craftItem.272");
        STATISTICS.put(16843025, "stat.craftItem.273");
        STATISTICS.put(16843026, "stat.craftItem.274");
        STATISTICS.put(16843027, "stat.craftItem.275");
        STATISTICS.put(16843028, "stat.craftItem.276");
        STATISTICS.put(16843029, "stat.craftItem.277");
        STATISTICS.put(16843030, "stat.craftItem.278");
        STATISTICS.put(16843031, "stat.craftItem.279");
        STATISTICS.put(16843032, "stat.craftItem.280");
        STATISTICS.put(16843033, "stat.craftItem.281");
        STATISTICS.put(16843034, "stat.craftItem.282");
        STATISTICS.put(16843035, "stat.craftItem.283");
        STATISTICS.put(16843036, "stat.craftItem.284");
        STATISTICS.put(16843037, "stat.craftItem.285");
        STATISTICS.put(16843038, "stat.craftItem.286");
        STATISTICS.put(16843042, "stat.craftItem.290");
        STATISTICS.put(16843043, "stat.craftItem.291");
        STATISTICS.put(16843044, "stat.craftItem.292");
        STATISTICS.put(16843045, "stat.craftItem.293");
        STATISTICS.put(16843046, "stat.craftItem.294");
        STATISTICS.put(16843048, "stat.craftItem.296");
        STATISTICS.put(16843049, "stat.craftItem.297");
        STATISTICS.put(16843050, "stat.craftItem.298");
        STATISTICS.put(16843051, "stat.craftItem.299");
        STATISTICS.put(16843052, "stat.craftItem.300");
        STATISTICS.put(16843053, "stat.craftItem.301");
        STATISTICS.put(16843054, "stat.craftItem.302");
        STATISTICS.put(16843055, "stat.craftItem.303");
        STATISTICS.put(16843056, "stat.craftItem.304");
        STATISTICS.put(16843057, "stat.craftItem.305");
        STATISTICS.put(16843058, "stat.craftItem.306");
        STATISTICS.put(16843059, "stat.craftItem.307");
        STATISTICS.put(16843060, "stat.craftItem.308");
        STATISTICS.put(16843061, "stat.craftItem.309");
        STATISTICS.put(16843062, "stat.craftItem.310");
        STATISTICS.put(16843063, "stat.craftItem.311");
        STATISTICS.put(16843064, "stat.craftItem.312");
        STATISTICS.put(16843065, "stat.craftItem.313");
        STATISTICS.put(16843066, "stat.craftItem.314");
        STATISTICS.put(16843067, "stat.craftItem.315");
        STATISTICS.put(16843068, "stat.craftItem.316");
        STATISTICS.put(16843069, "stat.craftItem.317");
        STATISTICS.put(16843072, "stat.craftItem.320");
        STATISTICS.put(16843073, "stat.craftItem.321");
        STATISTICS.put(16843074, "stat.craftItem.322");
        STATISTICS.put(16843075, "stat.craftItem.323");
        STATISTICS.put(16843076, "stat.craftItem.324");
        STATISTICS.put(16843077, "stat.craftItem.325");
        STATISTICS.put(16843080, "stat.craftItem.328");
        STATISTICS.put(16843082, "stat.craftItem.330");
        STATISTICS.put(16843083, "stat.craftItem.331");
        STATISTICS.put(16843085, "stat.craftItem.333");
        STATISTICS.put(16843088, "stat.craftItem.336");
        STATISTICS.put(16843091, "stat.craftItem.339");
        STATISTICS.put(16843092, "stat.craftItem.340");
        STATISTICS.put(16843094, "stat.craftItem.342");
        STATISTICS.put(16843095, "stat.craftItem.343");
        STATISTICS.put(16843097, "stat.craftItem.345");
        STATISTICS.put(16843098, "stat.craftItem.346");
        STATISTICS.put(16843099, "stat.craftItem.347");
        STATISTICS.put(16843102, "stat.craftItem.350");
        STATISTICS.put(16843103, "stat.craftItem.351");
        STATISTICS.put(16843105, "stat.craftItem.353");
        STATISTICS.put(16843106, "stat.craftItem.354");
        STATISTICS.put(16843107, "stat.craftItem.355");
        STATISTICS.put(16843108, "stat.craftItem.356");
        STATISTICS.put(16843109, "stat.craftItem.357");
        STATISTICS.put(16843111, "stat.craftItem.359");
        STATISTICS.put(16843113, "stat.craftItem.361");
        STATISTICS.put(16843114, "stat.craftItem.362");
        STATISTICS.put(16843116, "stat.craftItem.364");
        STATISTICS.put(16843118, "stat.craftItem.366");
        STATISTICS.put(16843123, "stat.craftItem.371");
        STATISTICS.put(16843126, "stat.craftItem.374");
        STATISTICS.put(16843128, "stat.craftItem.376");
        STATISTICS.put(16843129, "stat.craftItem.377");
        STATISTICS.put(16843130, "stat.craftItem.378");
        STATISTICS.put(16843131, "stat.craftItem.379");
        STATISTICS.put(16843132, "stat.craftItem.380");
        STATISTICS.put(16843133, "stat.craftItem.381");
        STATISTICS.put(16843134, "stat.craftItem.382");
        STATISTICS.put(16843137, "stat.craftItem.385");
        STATISTICS.put(16843138, "stat.craftItem.386");
        STATISTICS.put(16843140, "stat.craftItem.388");
        STATISTICS.put(16843141, "stat.craftItem.389");
        STATISTICS.put(16843142, "stat.craftItem.390");
        STATISTICS.put(16843145, "stat.craftItem.393");
        STATISTICS.put(16843147, "stat.craftItem.395");
        STATISTICS.put(16843148, "stat.craftItem.396");
        STATISTICS.put(16843150, "stat.craftItem.398");
        STATISTICS.put(16843152, "stat.craftItem.400");
        STATISTICS.put(16843156, "stat.craftItem.404");
        STATISTICS.put(16843157, "stat.craftItem.405");
        STATISTICS.put(16843158, "stat.craftItem.406");
        STATISTICS.put(16843159, "stat.craftItem.407");
        STATISTICS.put(16843160, "stat.craftItem.408");
        STATISTICS.put(16843172, "stat.craftItem.420");
        STATISTICS.put(16908289, "stat.useItem.1");
        STATISTICS.put(16908290, "stat.useItem.2");
        STATISTICS.put(16908291, "stat.useItem.3");
        STATISTICS.put(16908292, "stat.useItem.4");
        STATISTICS.put(16908293, "stat.useItem.5");
        STATISTICS.put(16908294, "stat.useItem.6");
        STATISTICS.put(16908295, "stat.useItem.7");
        STATISTICS.put(16908296, "stat.useItem.8");
        STATISTICS.put(16908297, "stat.useItem.9");
        STATISTICS.put(16908298, "stat.useItem.10");
        STATISTICS.put(16908299, "stat.useItem.11");
        STATISTICS.put(16908300, "stat.useItem.12");
        STATISTICS.put(16908301, "stat.useItem.13");
        STATISTICS.put(16908302, "stat.useItem.14");
        STATISTICS.put(16908303, "stat.useItem.15");
        STATISTICS.put(16908304, "stat.useItem.16");
        STATISTICS.put(16908305, "stat.useItem.17");
        STATISTICS.put(16908306, "stat.useItem.18");
        STATISTICS.put(16908307, "stat.useItem.19");
        STATISTICS.put(16908308, "stat.useItem.20");
        STATISTICS.put(16908309, "stat.useItem.21");
        STATISTICS.put(16908310, "stat.useItem.22");
        STATISTICS.put(16908311, "stat.useItem.23");
        STATISTICS.put(16908312, "stat.useItem.24");
        STATISTICS.put(16908313, "stat.useItem.25");
        STATISTICS.put(16908314, "stat.useItem.26");
        STATISTICS.put(16908315, "stat.useItem.27");
        STATISTICS.put(16908316, "stat.useItem.28");
        STATISTICS.put(16908317, "stat.useItem.29");
        STATISTICS.put(16908318, "stat.useItem.30");
        STATISTICS.put(16908319, "stat.useItem.31");
        STATISTICS.put(16908320, "stat.useItem.32");
        STATISTICS.put(16908321, "stat.useItem.33");
        STATISTICS.put(16908322, "stat.useItem.34");
        STATISTICS.put(16908323, "stat.useItem.35");
        STATISTICS.put(16908324, "stat.useItem.36");
        STATISTICS.put(16908325, "stat.useItem.37");
        STATISTICS.put(16908326, "stat.useItem.38");
        STATISTICS.put(16908327, "stat.useItem.39");
        STATISTICS.put(16908328, "stat.useItem.40");
        STATISTICS.put(16908329, "stat.useItem.41");
        STATISTICS.put(16908330, "stat.useItem.42");
        STATISTICS.put(16908331, "stat.useItem.43");
        STATISTICS.put(16908332, "stat.useItem.44");
        STATISTICS.put(16908333, "stat.useItem.45");
        STATISTICS.put(16908334, "stat.useItem.46");
        STATISTICS.put(16908335, "stat.useItem.47");
        STATISTICS.put(16908336, "stat.useItem.48");
        STATISTICS.put(16908337, "stat.useItem.49");
        STATISTICS.put(16908338, "stat.useItem.50");
        STATISTICS.put(16908339, "stat.useItem.51");
        STATISTICS.put(16908340, "stat.useItem.52");
        STATISTICS.put(16908341, "stat.useItem.53");
        STATISTICS.put(16908342, "stat.useItem.54");
        STATISTICS.put(16908343, "stat.useItem.55");
        STATISTICS.put(16908344, "stat.useItem.56");
        STATISTICS.put(16908345, "stat.useItem.57");
        STATISTICS.put(16908346, "stat.useItem.58");
        STATISTICS.put(16908347, "stat.useItem.59");
        STATISTICS.put(16908348, "stat.useItem.60");
        STATISTICS.put(16908349, "stat.useItem.61");
        STATISTICS.put(16908350, "stat.useItem.62");
        STATISTICS.put(16908351, "stat.useItem.63");
        STATISTICS.put(16908352, "stat.useItem.64");
        STATISTICS.put(16908353, "stat.useItem.65");
        STATISTICS.put(16908354, "stat.useItem.66");
        STATISTICS.put(16908355, "stat.useItem.67");
        STATISTICS.put(16908356, "stat.useItem.68");
        STATISTICS.put(16908357, "stat.useItem.69");
        STATISTICS.put(16908358, "stat.useItem.70");
        STATISTICS.put(16908359, "stat.useItem.71");
        STATISTICS.put(16908360, "stat.useItem.72");
        STATISTICS.put(16908361, "stat.useItem.73");
        STATISTICS.put(16908362, "stat.useItem.74");
        STATISTICS.put(16908363, "stat.useItem.75");
        STATISTICS.put(16908364, "stat.useItem.76");
        STATISTICS.put(16908365, "stat.useItem.77");
        STATISTICS.put(16908366, "stat.useItem.78");
        STATISTICS.put(16908367, "stat.useItem.79");
        STATISTICS.put(16908368, "stat.useItem.80");
        STATISTICS.put(16908369, "stat.useItem.81");
        STATISTICS.put(16908370, "stat.useItem.82");
        STATISTICS.put(16908371, "stat.useItem.83");
        STATISTICS.put(16908372, "stat.useItem.84");
        STATISTICS.put(16908373, "stat.useItem.85");
        STATISTICS.put(16908374, "stat.useItem.86");
        STATISTICS.put(16908375, "stat.useItem.87");
        STATISTICS.put(16908376, "stat.useItem.88");
        STATISTICS.put(16908377, "stat.useItem.89");
        STATISTICS.put(16908378, "stat.useItem.90");
        STATISTICS.put(16908379, "stat.useItem.91");
        STATISTICS.put(16908380, "stat.useItem.92");
        STATISTICS.put(16908381, "stat.useItem.93");
        STATISTICS.put(16908382, "stat.useItem.94");
        STATISTICS.put(16908383, "stat.useItem.95");
        STATISTICS.put(16908384, "stat.useItem.96");
        STATISTICS.put(16908385, "stat.useItem.97");
        STATISTICS.put(16908386, "stat.useItem.98");
        STATISTICS.put(16908387, "stat.useItem.99");
        STATISTICS.put(16908388, "stat.useItem.100");
        STATISTICS.put(16908389, "stat.useItem.101");
        STATISTICS.put(16908390, "stat.useItem.102");
        STATISTICS.put(16908391, "stat.useItem.103");
        STATISTICS.put(16908392, "stat.useItem.104");
        STATISTICS.put(16908393, "stat.useItem.105");
        STATISTICS.put(16908394, "stat.useItem.106");
        STATISTICS.put(16908395, "stat.useItem.107");
        STATISTICS.put(16908396, "stat.useItem.108");
        STATISTICS.put(16908397, "stat.useItem.109");
        STATISTICS.put(16908398, "stat.useItem.110");
        STATISTICS.put(16908399, "stat.useItem.111");
        STATISTICS.put(16908400, "stat.useItem.112");
        STATISTICS.put(16908401, "stat.useItem.113");
        STATISTICS.put(16908402, "stat.useItem.114");
        STATISTICS.put(16908403, "stat.useItem.115");
        STATISTICS.put(16908404, "stat.useItem.116");
        STATISTICS.put(16908405, "stat.useItem.117");
        STATISTICS.put(16908406, "stat.useItem.118");
        STATISTICS.put(16908407, "stat.useItem.119");
        STATISTICS.put(16908408, "stat.useItem.120");
        STATISTICS.put(16908409, "stat.useItem.121");
        STATISTICS.put(16908410, "stat.useItem.122");
        STATISTICS.put(16908411, "stat.useItem.123");
        STATISTICS.put(16908412, "stat.useItem.124");
        STATISTICS.put(16908413, "stat.useItem.125");
        STATISTICS.put(16908414, "stat.useItem.126");
        STATISTICS.put(16908415, "stat.useItem.127");
        STATISTICS.put(16908416, "stat.useItem.128");
        STATISTICS.put(16908417, "stat.useItem.129");
        STATISTICS.put(16908418, "stat.useItem.130");
        STATISTICS.put(16908419, "stat.useItem.131");
        STATISTICS.put(16908420, "stat.useItem.132");
        STATISTICS.put(16908421, "stat.useItem.133");
        STATISTICS.put(16908422, "stat.useItem.134");
        STATISTICS.put(16908423, "stat.useItem.135");
        STATISTICS.put(16908424, "stat.useItem.136");
        STATISTICS.put(16908425, "stat.useItem.137");
        STATISTICS.put(16908426, "stat.useItem.138");
        STATISTICS.put(16908427, "stat.useItem.139");
        STATISTICS.put(16908428, "stat.useItem.140");
        STATISTICS.put(16908429, "stat.useItem.141");
        STATISTICS.put(16908430, "stat.useItem.142");
        STATISTICS.put(16908431, "stat.useItem.143");
        STATISTICS.put(16908432, "stat.useItem.144");
        STATISTICS.put(16908433, "stat.useItem.145");
        STATISTICS.put(16908434, "stat.useItem.146");
        STATISTICS.put(16908435, "stat.useItem.147");
        STATISTICS.put(16908436, "stat.useItem.148");
        STATISTICS.put(16908437, "stat.useItem.149");
        STATISTICS.put(16908438, "stat.useItem.150");
        STATISTICS.put(16908439, "stat.useItem.151");
        STATISTICS.put(16908440, "stat.useItem.152");
        STATISTICS.put(16908441, "stat.useItem.153");
        STATISTICS.put(16908442, "stat.useItem.154");
        STATISTICS.put(16908443, "stat.useItem.155");
        STATISTICS.put(16908444, "stat.useItem.156");
        STATISTICS.put(16908445, "stat.useItem.157");
        STATISTICS.put(16908446, "stat.useItem.158");
        STATISTICS.put(16908447, "stat.useItem.159");
        STATISTICS.put(16908458, "stat.useItem.170");
        STATISTICS.put(16908459, "stat.useItem.171");
        STATISTICS.put(16908460, "stat.useItem.172");
        STATISTICS.put(16908461, "stat.useItem.173");
        STATISTICS.put(16908544, "stat.useItem.256");
        STATISTICS.put(16908545, "stat.useItem.257");
        STATISTICS.put(16908546, "stat.useItem.258");
        STATISTICS.put(16908547, "stat.useItem.259");
        STATISTICS.put(16908548, "stat.useItem.260");
        STATISTICS.put(16908549, "stat.useItem.261");
        STATISTICS.put(16908550, "stat.useItem.262");
        STATISTICS.put(16908551, "stat.useItem.263");
        STATISTICS.put(16908552, "stat.useItem.264");
        STATISTICS.put(16908553, "stat.useItem.265");
        STATISTICS.put(16908554, "stat.useItem.266");
        STATISTICS.put(16908555, "stat.useItem.267");
        STATISTICS.put(16908556, "stat.useItem.268");
        STATISTICS.put(16908557, "stat.useItem.269");
        STATISTICS.put(16908558, "stat.useItem.270");
        STATISTICS.put(16908559, "stat.useItem.271");
        STATISTICS.put(16908560, "stat.useItem.272");
        STATISTICS.put(16908561, "stat.useItem.273");
        STATISTICS.put(16908562, "stat.useItem.274");
        STATISTICS.put(16908563, "stat.useItem.275");
        STATISTICS.put(16908564, "stat.useItem.276");
        STATISTICS.put(16908565, "stat.useItem.277");
        STATISTICS.put(16908566, "stat.useItem.278");
        STATISTICS.put(16908567, "stat.useItem.279");
        STATISTICS.put(16908568, "stat.useItem.280");
        STATISTICS.put(16908569, "stat.useItem.281");
        STATISTICS.put(16908570, "stat.useItem.282");
        STATISTICS.put(16908571, "stat.useItem.283");
        STATISTICS.put(16908572, "stat.useItem.284");
        STATISTICS.put(16908573, "stat.useItem.285");
        STATISTICS.put(16908574, "stat.useItem.286");
        STATISTICS.put(16908575, "stat.useItem.287");
        STATISTICS.put(16908576, "stat.useItem.288");
        STATISTICS.put(16908577, "stat.useItem.289");
        STATISTICS.put(16908578, "stat.useItem.290");
        STATISTICS.put(16908579, "stat.useItem.291");
        STATISTICS.put(16908580, "stat.useItem.292");
        STATISTICS.put(16908581, "stat.useItem.293");
        STATISTICS.put(16908582, "stat.useItem.294");
        STATISTICS.put(16908583, "stat.useItem.295");
        STATISTICS.put(16908584, "stat.useItem.296");
        STATISTICS.put(16908585, "stat.useItem.297");
        STATISTICS.put(16908586, "stat.useItem.298");
        STATISTICS.put(16908587, "stat.useItem.299");
        STATISTICS.put(16908588, "stat.useItem.300");
        STATISTICS.put(16908589, "stat.useItem.301");
        STATISTICS.put(16908590, "stat.useItem.302");
        STATISTICS.put(16908591, "stat.useItem.303");
        STATISTICS.put(16908592, "stat.useItem.304");
        STATISTICS.put(16908593, "stat.useItem.305");
        STATISTICS.put(16908594, "stat.useItem.306");
        STATISTICS.put(16908595, "stat.useItem.307");
        STATISTICS.put(16908596, "stat.useItem.308");
        STATISTICS.put(16908597, "stat.useItem.309");
        STATISTICS.put(16908598, "stat.useItem.310");
        STATISTICS.put(16908599, "stat.useItem.311");
        STATISTICS.put(16908600, "stat.useItem.312");
        STATISTICS.put(16908601, "stat.useItem.313");
        STATISTICS.put(16908602, "stat.useItem.314");
        STATISTICS.put(16908603, "stat.useItem.315");
        STATISTICS.put(16908604, "stat.useItem.316");
        STATISTICS.put(16908605, "stat.useItem.317");
        STATISTICS.put(16908606, "stat.useItem.318");
        STATISTICS.put(16908607, "stat.useItem.319");
        STATISTICS.put(16908608, "stat.useItem.320");
        STATISTICS.put(16908609, "stat.useItem.321");
        STATISTICS.put(16908610, "stat.useItem.322");
        STATISTICS.put(16908611, "stat.useItem.323");
        STATISTICS.put(16908612, "stat.useItem.324");
        STATISTICS.put(16908613, "stat.useItem.325");
        STATISTICS.put(16908614, "stat.useItem.326");
        STATISTICS.put(16908615, "stat.useItem.327");
        STATISTICS.put(16908616, "stat.useItem.328");
        STATISTICS.put(16908617, "stat.useItem.329");
        STATISTICS.put(16908618, "stat.useItem.330");
        STATISTICS.put(16908619, "stat.useItem.331");
        STATISTICS.put(16908620, "stat.useItem.332");
        STATISTICS.put(16908621, "stat.useItem.333");
        STATISTICS.put(16908622, "stat.useItem.334");
        STATISTICS.put(16908623, "stat.useItem.335");
        STATISTICS.put(16908624, "stat.useItem.336");
        STATISTICS.put(16908625, "stat.useItem.337");
        STATISTICS.put(16908626, "stat.useItem.338");
        STATISTICS.put(16908627, "stat.useItem.339");
        STATISTICS.put(16908628, "stat.useItem.340");
        STATISTICS.put(16908629, "stat.useItem.341");
        STATISTICS.put(16908630, "stat.useItem.342");
        STATISTICS.put(16908631, "stat.useItem.343");
        STATISTICS.put(16908632, "stat.useItem.344");
        STATISTICS.put(16908633, "stat.useItem.345");
        STATISTICS.put(16908634, "stat.useItem.346");
        STATISTICS.put(16908635, "stat.useItem.347");
        STATISTICS.put(16908636, "stat.useItem.348");
        STATISTICS.put(16908637, "stat.useItem.349");
        STATISTICS.put(16908638, "stat.useItem.350");
        STATISTICS.put(16908639, "stat.useItem.351");
        STATISTICS.put(16908640, "stat.useItem.352");
        STATISTICS.put(16908641, "stat.useItem.353");
        STATISTICS.put(16908642, "stat.useItem.354");
        STATISTICS.put(16908643, "stat.useItem.355");
        STATISTICS.put(16908644, "stat.useItem.356");
        STATISTICS.put(16908645, "stat.useItem.357");
        STATISTICS.put(16908646, "stat.useItem.358");
        STATISTICS.put(16908647, "stat.useItem.359");
        STATISTICS.put(16908648, "stat.useItem.360");
        STATISTICS.put(16908649, "stat.useItem.361");
        STATISTICS.put(16908650, "stat.useItem.362");
        STATISTICS.put(16908651, "stat.useItem.363");
        STATISTICS.put(16908652, "stat.useItem.364");
        STATISTICS.put(16908653, "stat.useItem.365");
        STATISTICS.put(16908654, "stat.useItem.366");
        STATISTICS.put(16908655, "stat.useItem.367");
        STATISTICS.put(16908656, "stat.useItem.368");
        STATISTICS.put(16908657, "stat.useItem.369");
        STATISTICS.put(16908658, "stat.useItem.370");
        STATISTICS.put(16908659, "stat.useItem.371");
        STATISTICS.put(16908660, "stat.useItem.372");
        STATISTICS.put(16908661, "stat.useItem.373");
        STATISTICS.put(16908662, "stat.useItem.374");
        STATISTICS.put(16908663, "stat.useItem.375");
        STATISTICS.put(16908664, "stat.useItem.376");
        STATISTICS.put(16908665, "stat.useItem.377");
        STATISTICS.put(16908666, "stat.useItem.378");
        STATISTICS.put(16908667, "stat.useItem.379");
        STATISTICS.put(16908668, "stat.useItem.380");
        STATISTICS.put(16908669, "stat.useItem.381");
        STATISTICS.put(16908670, "stat.useItem.382");
        STATISTICS.put(16908671, "stat.useItem.383");
        STATISTICS.put(16908672, "stat.useItem.384");
        STATISTICS.put(16908673, "stat.useItem.385");
        STATISTICS.put(16908674, "stat.useItem.386");
        STATISTICS.put(16908675, "stat.useItem.387");
        STATISTICS.put(16908676, "stat.useItem.388");
        STATISTICS.put(16908677, "stat.useItem.389");
        STATISTICS.put(16908678, "stat.useItem.390");
        STATISTICS.put(16908679, "stat.useItem.391");
        STATISTICS.put(16908680, "stat.useItem.392");
        STATISTICS.put(16908681, "stat.useItem.393");
        STATISTICS.put(16908682, "stat.useItem.394");
        STATISTICS.put(16908683, "stat.useItem.395");
        STATISTICS.put(16908684, "stat.useItem.396");
        STATISTICS.put(16908685, "stat.useItem.397");
        STATISTICS.put(16908686, "stat.useItem.398");
        STATISTICS.put(16908687, "stat.useItem.399");
        STATISTICS.put(16908688, "stat.useItem.400");
        STATISTICS.put(16908689, "stat.useItem.401");
        STATISTICS.put(16908690, "stat.useItem.402");
        STATISTICS.put(16908691, "stat.useItem.403");
        STATISTICS.put(16908692, "stat.useItem.404");
        STATISTICS.put(16908693, "stat.useItem.405");
        STATISTICS.put(16908694, "stat.useItem.406");
        STATISTICS.put(16908695, "stat.useItem.407");
        STATISTICS.put(16908696, "stat.useItem.408");
        STATISTICS.put(16908705, "stat.useItem.417");
        STATISTICS.put(16908706, "stat.useItem.418");
        STATISTICS.put(16908707, "stat.useItem.419");
        STATISTICS.put(16908708, "stat.useItem.420");
        STATISTICS.put(16908709, "stat.useItem.421");
        STATISTICS.put(16910544, "stat.useItem.2256");
        STATISTICS.put(16910545, "stat.useItem.2257");
        STATISTICS.put(16910546, "stat.useItem.2258");
        STATISTICS.put(16910547, "stat.useItem.2259");
        STATISTICS.put(16910548, "stat.useItem.2260");
        STATISTICS.put(16910549, "stat.useItem.2261");
        STATISTICS.put(16910550, "stat.useItem.2262");
        STATISTICS.put(16910551, "stat.useItem.2263");
        STATISTICS.put(16910552, "stat.useItem.2264");
        STATISTICS.put(16910553, "stat.useItem.2265");
        STATISTICS.put(16910554, "stat.useItem.2266");
        STATISTICS.put(16910555, "stat.useItem.2267");
        STATISTICS.put(16974080, "stat.breakItem.256");
        STATISTICS.put(16974081, "stat.breakItem.257");
        STATISTICS.put(16974082, "stat.breakItem.258");
        STATISTICS.put(16974083, "stat.breakItem.259");
        STATISTICS.put(16974085, "stat.breakItem.261");
        STATISTICS.put(16974091, "stat.breakItem.267");
        STATISTICS.put(16974092, "stat.breakItem.268");
        STATISTICS.put(16974093, "stat.breakItem.269");
        STATISTICS.put(16974094, "stat.breakItem.270");
        STATISTICS.put(16974095, "stat.breakItem.271");
        STATISTICS.put(16974096, "stat.breakItem.272");
        STATISTICS.put(16974097, "stat.breakItem.273");
        STATISTICS.put(16974098, "stat.breakItem.274");
        STATISTICS.put(16974099, "stat.breakItem.275");
        STATISTICS.put(16974100, "stat.breakItem.276");
        STATISTICS.put(16974101, "stat.breakItem.277");
        STATISTICS.put(16974102, "stat.breakItem.278");
        STATISTICS.put(16974103, "stat.breakItem.279");
        STATISTICS.put(16974107, "stat.breakItem.283");
        STATISTICS.put(16974108, "stat.breakItem.284");
        STATISTICS.put(16974109, "stat.breakItem.285");
        STATISTICS.put(16974110, "stat.breakItem.286");
        STATISTICS.put(16974114, "stat.breakItem.290");
        STATISTICS.put(16974115, "stat.breakItem.291");
        STATISTICS.put(16974116, "stat.breakItem.292");
        STATISTICS.put(16974117, "stat.breakItem.293");
        STATISTICS.put(16974118, "stat.breakItem.294");
        STATISTICS.put(16974122, "stat.breakItem.298");
        STATISTICS.put(16974123, "stat.breakItem.299");
        STATISTICS.put(16974124, "stat.breakItem.300");
        STATISTICS.put(16974125, "stat.breakItem.301");
        STATISTICS.put(16974126, "stat.breakItem.302");
        STATISTICS.put(16974127, "stat.breakItem.303");
        STATISTICS.put(16974128, "stat.breakItem.304");
        STATISTICS.put(16974129, "stat.breakItem.305");
        STATISTICS.put(16974130, "stat.breakItem.306");
        STATISTICS.put(16974131, "stat.breakItem.307");
        STATISTICS.put(16974132, "stat.breakItem.308");
        STATISTICS.put(16974133, "stat.breakItem.309");
        STATISTICS.put(16974134, "stat.breakItem.310");
        STATISTICS.put(16974135, "stat.breakItem.311");
        STATISTICS.put(16974136, "stat.breakItem.312");
        STATISTICS.put(16974137, "stat.breakItem.313");
        STATISTICS.put(16974138, "stat.breakItem.314");
        STATISTICS.put(16974139, "stat.breakItem.315");
        STATISTICS.put(16974140, "stat.breakItem.316");
        STATISTICS.put(16974141, "stat.breakItem.317");
        STATISTICS.put(16974170, "stat.breakItem.346");
        STATISTICS.put(16974183, "stat.breakItem.359");
        STATISTICS.put(16974222, "stat.breakItem.398");
    }

    public static String map(final int stat) {
        return STATISTICS.get(stat);
    }

}